/*
  mainwindow.h
  Copyright (C) 2008-2009 Tomohiro Masubuchi <bucchi@e-ml.jp>

  This file is part of qgmap.

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __MAINWINDOW_H__
#define __MAINWINDOW_H__

#include "layeredbase.h"
#include "qgconf.h"
#include "qobjectlist.h"
#include <qpixmap.h>
#include <qstring.h>

class QWidget;
class Pos;

class Help;
class Info;
class IconMenu;
class CenterPos;
class PoiInfoView;

class Mounter;

#ifndef QWS
class QLabel;
#endif

class MainWindow : public LayeredBase
{
	Q_OBJECT
  public:
	MainWindow( QWidget *parent =0, const char *name = 0, WFlags fl = 0);
	~MainWindow();

	void dispInfo(QString str);
	void hideInfo();

	void dispPoiInfo(QPixmap const &icon, QString const &title,
					 QString const &desc, QString const &info);

	void showEvent( QShowEvent *e );
	void mousePressEvent( QMouseEvent *e );
	void keyPressEvent ( QKeyEvent * e );
	void paintEvent( QPaintEvent *e );

  private:
	QGConf conf;

	LayeredWidget *gmap;
	LayeredWidget *slider;
	LayeredWidget *marker;
	LayeredWidget *menu;
	LayeredWidget *scale;
	LayeredWidget *gpsview;
	LayeredWidget *poiview;
	LayeredWidget *centermarker;
	LayeredWidget *freehand;

	Help *help;
	Info *info;
	PoiInfoView *poiinfo;

	CenterPos *center;

	Mounter *mounter;

#ifndef QWS
	QLabel *label;
#endif

  public slots:
	void cleanupEventFilter() {}
	void recvQCop(const QCString &m, const QByteArray &d);

  signals:
	void destroyed();

	void zoomChanged(int value);
	void setPosition(Pos &pos);
	void positionChanged(Pos &pos);
	void configChanged(const QString &name, const QString &value);
	void reload(bool);
	void cacheFlush();
	void updateMap();
	void movePosition(Pos &pos);
};

static inline void *lookupObject(QObject *obj, const char *type, const QString &name)
{
	QObject *parent = obj;

	while(parent->parent())
		parent = parent->parent();

	return qt_find_obj_child(parent, type, name.ascii());
}

#endif // __MAINWINDOW_H__
