/*
  map_poi.cpp
  Copyright (C) 2008-2009 Tomohiro Masubuchi <bucchi@e-ml.jp>

  This file is part of qgmap.

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; If not, see <http://www.gnu.org/licenses/>.
*/

//#define DEBUG
#include "qgmap.h"
#include <qpixmap.h>
#include <qobject.h>
#include <qpainter.h>
#include <math.h>
#include "pos.h"
#include "map.h"
#include "map_poi.h"

#include <sqlite3.h>

MapItemPoi::MapItemPoi(Map *parent) :
	icon_cache(7, 7)
{
	Enter();
	parent->addItem(this);

//	connect(parent, SIGNAL(configChanged(const QString &, const QString &)),
//			this, SLOT(setConfig(const QString &, const QString &)));

	db = new PoiDb(DB_POI_PATH);

	cluster = QImage(APP_PATH "/cluster.png");
	icon_cache.setAutoDelete(true);
}

MapItemPoi::~MapItemPoi()
{
	delete db;
	db = NULL;
}

bool MapItemPoi::drawItem(QImage &scr, const Pos &pos)
{
	Enter();
	bool too_many;

	PoiDbItem *item = db->queryPoiMapItem(pos.getX(), pos.getY(), pos.getZoom(), POI_MAX, &too_many);
	if (!item) {
		if (too_many) {
			drawImage(scr, &cluster, IMAGE_SIZE * 3 / 4 - cluster.width() / 2, IMAGE_SIZE / 4 - cluster.height()/2);
			drawImage(scr, &cluster, IMAGE_SIZE / 4 - cluster.width() / 2, IMAGE_SIZE * 3 / 4 - cluster.height()/2);
		}

		return true;
	}

//	QPainter p(&scr);

	while (item->next()) {
		QString key = "ICON:%1";
		QImage *icon;
		int icon_id = item->getIconId();

#if 0
		printf("%d %d,%d Icon:%d %s\n",
			   item->getId(), item->getX(), item->getY(), item->getIconId(),
			   (const char *)item->getName().utf8());
#endif

		icon = icon_cache.find(key.arg(icon_id));
		if (!icon) {
			QImage tmp(db->getPoiIcon(icon_id));

			if (tmp.isNull())
				continue;

			icon = new QImage(tmp.convertDepth(32));

			icon_cache.insert(key.arg(icon_id), icon);
		}

		drawImage(scr, icon, item->getX() - icon->width() / 2, item->getY() - icon->height() / 2);

#if 0
		int x, y;
		printf("icon.depth() = %d\n", icon->depth());
		for (y = 0; y < icon->height(); y++) {
			unsigned int *scan = (unsigned int *)icon->scanLine(y);
			for (x = 0; x < icon->width(); x++) {
				printf("%02x ", (*scan) >> 24);
				scan++;
			}
			printf("\n");
		}
#endif
	}

	delete item;

//	printf("memory usage: %lld\n", sqlite3_memory_used());

	return true;
}

#if 0
void MapItemPoi::setConfig(const QString &key, const QString &value)
{
	Enter();
}
#endif
